// *********************************************************************
//
// BrianHG_DDR3_DECA_test1_top_tb DDR3 sequencer.
// Version 0.5, March 10, 2021.
//
//
// Written by Brian Guralnick.
// For public use.
// Leave questions in the https://www.eevblog.com/forum/fpga/brianhg_ddr3_controller-open-source-ddr3-controller/
//
//************************************************************************************************************************************************************
//************************************************************************************************************************************************************
//************************************************************************************************************************************************************
//*** DDR3 Verilog model from Micron Required for this test-bench.
//*** The required DDR3 SDRAM Verilog Model V1.74 available at:
//*** https://media-www.micron.com/-/media/client/global/documents/products/sim-model/dram/ddr3/ddr3-sdram-verilog-model.zip?rev=925a8a05204e4b5c9c1364302de60126
//*** From the 'DDR3 SDRAM Verilog Model.zip', only these 2 files are required in the main simulation test-bench source folder:
//*** ddr3.v
//*** 4096Mb_ddr3_parameters.vh
//************************************************************************************************************************************************************
// Tell Micron's DDR3 Verilog model which ram chip we expect to have connected to the test bench.
//************************************************************************************************************************************************************
`define den4096Mb
`define sg093
`define x16
//************************************************************************************************************************************************************
//************************************************************************************************************************************************************
//************************************************************************************************************************************************************
`timescale 1 ps/ 1 ps // 1 picosecond steps, 1 picosecond precision.

module BrianHG_DDR3_DECA_test1_top_tb #(

parameter string     FPGA_VENDOR             = "Altera",         // (Only Altera for now) Use ALTERA, INTEL, LATTICE or XILINX.
parameter string     FPGA_FAMILY             = "MAX 10",         // With Altera, use Cyclone III, Cyclone IV, Cyclone V, MAX 10,....

// ****************  System clock generation and operation.
parameter int        CLK_KHZ_IN              = 50500,            // PLL source input clock frequency in KHz.
parameter int        CLK_IN_MULT             = 30,               // Multiply factor to generate the DDR MTPS speed divided by 2.
parameter int        CLK_IN_DIV              = 5,                // Divide factor to generate the DDR MTPS speed divided by 2.
parameter int        DDR_TRICK_MTPS_CAP      = 0,                // 0=off, Set a false PLL DDR data rate for the compiler to allow FPGA overclocking.  ***DO NOT USE.
parameter bit        ENABLE_RS232_DIAG       = 1,                // 1=on, 0=off, Enables real-time RS232 diagnostic port read-timing analysis port.
parameter bit        READ_PHASE              = 0,                // Enter the value attained from the RS232 diagnostic tool during a full temperature sweep for best operation
                                                                 // or use 0 for auto read alignment during power-up and reset.
                                                                 // (not yet supported, -1 for real-time continuous read-leveling during operation)
parameter int        ENABLE_RAM_TEST         = 1,                // After power-up or reset, perform a ram read/write test.
                                                                 // 0=disable, 1=test every byte, 128=test beginning every row and bank.
                                                                 // *** Testing every single byte on a 512 megabyte device can take ~5 full seconds.
                                                                
parameter string     INTERFACE_SPEED         = "Quarter",        // Either "Half", "Quarter", or "Eighth" speed for the user interface clock.
                                                                 // This will effect the controller's interface CMD_CLK output port frequency.

// ****************  DDR3 ram chip configuration settings
parameter int        DDR3_CK_MHZ             = ((CLK_KHZ_IN*CLK_IN_MULT/CLK_IN_DIV)/1000), // DDR3 CK clock speed in MHz.
parameter string     DDR3_SPEED_GRADE        = "-093",           // Use 1066 / 187E, 1333 / -15E, 1600 / -125, 1866 / -107, or 2133 MHz / 093.
parameter int        DDR3_SIZE_GB            = 4,                // Use 0,1,2,4 or 8.  (0=512mb) Caution: Must be correct as ram chip size affects the tRFC REFRESH period.
parameter int        DDR3_WIDTH_DQ           = 16,               // Use 8 or 16.  The width of each DDR3 ram chip.

parameter int        DDR3_NUM_CHIPS          = 1,                // 1, 2, or 4 for the number of DDR3 RAM chips.
parameter int        DDR3_NUM_CK             = (DDR3_NUM_CHIPS), // Select the number of DDR3_CK & DDR3_CK# output pairs.  Add 1 for every DDR3 Ram chip.
                                                                 // These are placed on a DDR DQ or DDR CK# IO output pins.

parameter int        DDR3_WIDTH_ADDR         = 15,               // Use for the number of bits to address each row.
parameter int        DDR3_WIDTH_BANK         = 3,                // Use for the number of bits to address each bank.
parameter int        DDR3_WIDTH_CAS          = 10,               // Use for the number of bits to address each column.

parameter int        DDR3_WIDTH_DM           = (DDR3_WIDTH_DQ*DDR3_NUM_CHIPS/8), // The width of the write data mask. (***Double when using multiple 4 bit DDR3 ram chips.)
parameter int        DDR3_WIDTH_DQS          = (DDR3_WIDTH_DQ*DDR3_NUM_CHIPS/8), // The number of DQS pairs.          (***Double when using multiple 4 bit DDR3 ram chips.)
parameter int        DDR3_RWDQ_BITS          = (DDR3_WIDTH_DQ*DDR3_NUM_CHIPS*8), // Must equal to total bus width across all DDR3 ram chips *8.

parameter int        DDR3_ODT_RTT            = 120,              // use 120, 60, 40, 30, 20 Ohm. or 0 to disable ODT.  (On Die Termination during write operation.)
parameter int        DDR3_RZQ                = 40,               // use 34 or 40 Ohm. (Output Drive Strength during read operation.)
parameter int        DDR3_TEMP               = 85,               // use 85,95,105. (Peak operating temperature in degrees Celsius.)

parameter int        DDR3_WDQ_PHASE          = 90,               // 90, 0,45,90,...270 Select the write and write DQS output clock phase relative to the DDR3_CK/CK#
parameter int        DDR3_RDQ_PHASE          = 180,               // 45, 0,45,90,...270 Select the read latch clock for the read data and DQS input relative to the DDR3_CK.

parameter bit [3:0]  DDR3_MAX_REF_QUEUE      = 8,                // Defines the size of the refresh queue where refreshes will have a higher priority than incoming SEQ_CMD_ENA command requests.
                                                                 // *** Do not go above 8, doing so may break the data sheet's maximum ACTIVATE-to-PRECHARGE command period as a
parameter bit [6:0]  IDLE_TIME_uSx10         = 10,               // Defines the time in 1/10uS until the command IDLE counter will allow low priority REFRESH cycles.
                                                                 // Use 10 for 1uS.  0=disable, 2 for a minimum effect, 127 maximum.

parameter int        POWR_UP_TIMER_uS        = 1,//600,              // Timer during and after reset.  500 minimum, 600 default, 800 max, ***1 for quick simulations only.

parameter string     BANK_ROW_ORDER          = "ROW_BANK_COL",   // Only supports "ROW_BANK_COL" or "BANK_ROW_COL".  Choose to optimize your memory access.

// ****************  DDR3 controller configuration parameter settings.
parameter int        PORT_VECTOR_SIZE        = 16,               // Set the width of the SEQ_RDATA_VECT_IN & SEQ_RDATA_VECT_OUT port, 1 through 64.
parameter int        PORT_ADDR_SIZE          = (DDR3_WIDTH_ADDR + DDR3_WIDTH_BANK + DDR3_WIDTH_CAS + (DDR3_WIDTH_DM-1))
)
(
ADC_CLK_10,
MAX10_CLK1_50,
MAX10_CLK2_50,
LED,
RS232_RXD,
RS232_TXD,
DDR3_RESET_n,
DDR3_CK_p,   
DDR3_CK_n,   
DDR3_CKE,    
DDR3_CS_n,   
DDR3_RAS_n,  
DDR3_CAS_n,  
DDR3_WE_n,   
DDR3_ODT,    
DDR3_A,      
DDR3_BA,     
DDR3_DM,     
DDR3_DQ,     
DDR3_DQS_p,  
DDR3_DQS_n );


input  logic              ADC_CLK_10                 ;
input  logic              MAX10_CLK1_50              ;
input  logic              MAX10_CLK2_50              ;
output logic       [7:0]  LED                        ;
input  logic              RS232_RXD                  ;
output logic              RS232_TXD                  ;
output logic                        DDR3_RESET_n     ;
inout  logic  [DDR3_NUM_CK-1:0]     DDR3_CK_p        ;
inout  logic  [DDR3_NUM_CK-1:0]     DDR3_CK_n        ;
output logic                        DDR3_CKE         ;
output logic                        DDR3_CS_n        ;
output logic                        DDR3_RAS_n       ;
output logic                        DDR3_CAS_n       ;
output logic                        DDR3_WE_n        ;
output logic                        DDR3_ODT         ;
output logic  [DDR3_WIDTH_ADDR-1:0] DDR3_A           ;
output logic  [DDR3_WIDTH_BANK-1:0] DDR3_BA          ;
inout  logic  [DDR3_WIDTH_DM-1:0]   DDR3_DM          ;
inout  logic  [DDR3_WIDTH_DQ-1:0]   DDR3_DQ          ;
inout  logic  [DDR3_WIDTH_DQS-1:0]  DDR3_DQS_p       ;
inout  logic  [DDR3_WIDTH_DQS-1:0]  DDR3_DQS_n       ;

wire                [43:0]  GPIO0_D ;

BrianHG_DDR3_DECA_test1_top  BHG_DDR3_DECA_test1_top (

.ADC_CLK_10     (ADC_CLK_10     ),
.MAX10_CLK1_50  (MAX10_CLK1_50  ),
.MAX10_CLK2_50  (MAX10_CLK2_50  ),
.LED            (LED            ),
.GPIO0_D        (GPIO0_D        ),
.DDR3_RESET_n   (DDR3_RESET_n   ),
.DDR3_CK_p      (DDR3_CK_p      ),
.DDR3_CK_n      (DDR3_CK_n      ),
.DDR3_CKE       (DDR3_CKE       ),
.DDR3_CS_n      (DDR3_CS_n      ),
.DDR3_RAS_n     (DDR3_RAS_n     ),
.DDR3_CAS_n     (DDR3_CAS_n     ),
.DDR3_WE_n      (DDR3_WE_n      ),
.DDR3_ODT       (DDR3_ODT       ),
.DDR3_A         (DDR3_A         ),
.DDR3_BA        (DDR3_BA        ),
.DDR3_DM        (DDR3_DM        ),
.DDR3_DQ        (DDR3_DQ        ),
.DDR3_DQS_p     (DDR3_DQS_p     ),
.DDR3_DQS_n     (DDR3_DQS_n     ) );

assign RS232_TXD  = GPIO0_D[1];
assign GPIO0_D[3] = RS232_TXD;


localparam      period   = 500000000/CLK_KHZ_IN ;
localparam      STOP_uS  = 1000000 ;
localparam      endtime  = STOP_uS * 10;

//************************************************************************************************************************************************************
//*** DDR3 Verilog model from Micron Required for this test-bench.
//************************************************************************************************************************************************************
`include "ddr3.v"
//************************************************************************************************************************************************************
//*** DDR3 Verilog model from Micron Required for this test-bench.
//*** The required DDR3 SDRAM Verilog Model V1.74 available at:
//*** https://media-www.micron.com/-/media/client/global/documents/products/sim-model/dram/ddr3/ddr3-sdram-verilog-model.zip?rev=925a8a05204e4b5c9c1364302de60126
//*** From the 'DDR3 SDRAM Verilog Model.zip', only these 2 files are required in the main simulation test-bench source folder:
//*** ddr3.v
//*** 4096Mb_ddr3_parameters.vh
//************************************************************************************************************************************************************
    // component instantiation
    ddr3 sdramddr3_0 (
        .rst_n      ( DDR3_RESET_n ),
        .ck         ( DDR3_CK_p[0] ),
        .ck_n       ( DDR3_CK_n[0] ),
        .cke        ( DDR3_CKE     ),
        .cs_n       ( DDR3_CS_n    ),
        .ras_n      ( DDR3_RAS_n   ),
        .cas_n      ( DDR3_CAS_n   ),
        .we_n       ( DDR3_WE_n    ),
        .dm_tdqs    ( DDR3_DM      ),
        .ba         ( DDR3_BA      ),
        .addr       ( DDR3_A       ),
        .dq         ( DDR3_DQ      ),
        .dqs        ( DDR3_DQS_p   ),
        .dqs_n      ( DDR3_DQS_n   ),
        .tdqs_n     (              ),
        .odt        ( DDR3_ODT     )
    );
//************************************************************************************************************************************************************
//************************************************************************************************************************************************************
//************************************************************************************************************************************************************

logic      [11:0] WDT_COUNTER;                                                       // Wait for 15 clocks or inactivity before forcing a simulation stop.
logic             WAIT_IDLE        = 0;                                              // When high, insert a idle wait before every command.
localparam int    WDT_RESET_TIME   = 4000;                                             // Set the WDT timeout clock cycles.
localparam int    SYS_IDLE_TIME    = WDT_RESET_TIME-2000;                               // Consider system idle after 12 clocks of inactivity.
localparam real   DDR3_CK_MHZ_REAL = CLK_KHZ_IN * CLK_IN_MULT / CLK_IN_DIV / 1000 ;  // Generate the DDR3 CK clock frequency.
localparam real   DDR3_CK_PERIOD   = 1000 / DDR3_CK_MHZ_REAL ;                       // Generate the DDR3 CK period in nanoseconds.

initial begin
WDT_COUNTER       = WDT_RESET_TIME  ; // Set the initial inactivity timer to maximum so that the code later-on wont immediately stop the simulation.
ADC_CLK_10        = 0 ;
MAX10_CLK1_50     = 0 ;
MAX10_CLK2_50     = 0 ;
RS232_RXD         = 1 ; // clear RXD input

//RST_IN = 1'b1 ; // Reset input
//CLK_IN = 1'b0 ;
#(50000);
//RST_IN = 1'b0 ; // Release reset at 50ns.


end

always #period                  MAX10_CLK1_50 = !MAX10_CLK1_50;                                             // create source clock oscillator
always #period                  MAX10_CLK2_50 = !MAX10_CLK2_50;                                             // create source clock oscillator
always @(posedge MAX10_CLK1_50) WDT_COUNTER   = (DDR3_RESET_n==0 || DDR3_CAS_n==0) ?  WDT_RESET_TIME : (WDT_COUNTER-1'b1) ;   // Setup a simulation inactivity watchdog countdown timer.
always @(posedge MAX10_CLK1_50) if (WDT_COUNTER==0) $stop;                                           // Automatically stop the simulation if the inactivity timer reaches 0.


endmodule
